@@ -122,6 +122,7 @@ userconfig::userconfig(const std::string& name,
     , low_quality_score(2)
     , trim_ambiguous_bases(false)
     , max_ambiguous_bases(1000)
+    , preserve5p(false)
     , collapse(false)
     , deterministic(false)
     , shift(2)
@@ -352,6 +353,14 @@ userconfig::userconfig(const std::string& name,
         new argparse::knob(&low_quality_score, "PHRED",
             "Inclusive minimum; see --trimqualities for details "
             "[default: %default]");
+    argparser["--preserve5p"] =
+        new argparse::flag(&preserve5p,
+            "If set, bases at the 5p will not be trimmed by --trimns, "
+            "--trimqualities, and --trimwindows. Collapsed reads will "
+            "not be quality trimmed when this option is enabled "
+            "[default: 5p bases are trimmed]");
+
+    argparser.add_seperator();
     argparser["--minlength"] =
         new argparse::knob(&min_genomic_length, "LENGTH",
             "Reads shorter than this length are discarded "
@@ -689,13 +698,16 @@ std::string userconfig::get_output_filename(const std::string& key,
                                             size_t nth) const
 {
     std::string filename = basename;
+    if (filename.length() && filename.back() != '/') {
+        filename.push_back('.');
+    }
 
     if (key == "demux_stats") {
-        return filename += ".settings";
+        return filename += "settings";
     } else if (key == "demux_unknown") {
-        filename += ".unidentified";
+        filename += "unidentified";
 
-        AR_DEBUG_ASSERT(nth <= 9);
+        AR_DEBUG_ASSERT(nth <= 2);
         if (!interleaved_output) {
             if (nth) {
                 filename.push_back('_');
@@ -704,55 +716,42 @@ std::string userconfig::get_output_filename(const std::string& key,
         } else {
             filename += ".paired";
         }
-
-        // Currently only when demultiplexing; for backwards compatibility
-        if (run_type == ar_command::demultiplex_sequences) {
-            filename += ".fastq";
-        }
-
-        if (gzip) {
-            filename += ".gz";
-        } else if (bzip2) {
-            filename += ".bz2";
-        }
-
-        return filename;
     } else if (adapters.barcode_count()) {
-        filename.push_back('.');
         filename.append(adapters.get_sample_name(nth));
+        filename.push_back('.');
     } else if (argparser.is_set(key)) {
         return argparser.at(key)->to_str();
     }
 
     if (key == "--settings") {
-        return filename + ".settings";
+        return filename + "settings";
     } else if (key == "--outputcollapsed") {
-        filename += ".collapsed";
+        filename += "collapsed";
     } else if (key == "--outputcollapsedtruncated") {
-        filename += ".collapsed.truncated";
+        filename += "collapsed.truncated";
     } else if (key == "--discarded") {
-        filename += ".discarded";
+        filename += "discarded";
     } else if (paired_ended_mode) {
         if (key == "--output1") {
             if (interleaved_output) {
-                filename += ".paired";
+                filename += "paired";
             } else {
-                filename += ".pair1";
+                filename += "pair1";
             }
         } else if (key == "--output2") {
-            filename += ".pair2";
+            filename += "pair2";
         } else if (key == "--singleton") {
-            filename += ".singleton";
-        } else {
+            filename += "singleton";
+        } else if (key != "demux_unknown") {
             throw std::invalid_argument("invalid read-type in userconfig::get_output_filename constructor: " + key);
         }
 
-        if (run_type != ar_command::demultiplex_sequences) {
+        if (run_type != ar_command::demultiplex_sequences && key != "demux_unknown") {
             filename += ".truncated";
         }
     } else if (key == "--output1") {
         if (run_type != ar_command::demultiplex_sequences) {
-            filename += ".truncated";
+            filename += "truncated";
         }
     } else if (key != "demux_unknown") {
         throw std::invalid_argument("invalid read-type in userconfig::get_output_filename constructor: " + key);
@@ -760,7 +759,11 @@ std::string userconfig::get_output_filename(const std::string& key,
 
     // Currently only when demultiplexing; for backwards compatibility
     if (run_type == ar_command::demultiplex_sequences) {
-        filename += ".fastq";
+        if (filename.back() == '.') {
+            filename += "fastq";
+        } else {
+            filename += ".fastq";
+        }
     }
 
     if (gzip) {
