@@ -57,6 +57,34 @@ private:
 };
 
 
+///////////////////////////////////////////////////////////////////////////////
+// Helper functions
+
+TEST_CASE("ACGT_TO_IDX", "[fastq::*]")
+{
+    // The exact encoding is unimportant, but it must be unique and 2 bit
+    REQUIRE(ACGT_TO_IDX('A') <= 3);
+    REQUIRE(ACGT_TO_IDX('C') <= 3);
+    REQUIRE(ACGT_TO_IDX('G') <= 3);
+    REQUIRE(ACGT_TO_IDX('T') <= 3);
+
+    REQUIRE(ACGT_TO_IDX('A') != ACGT_TO_IDX('C'));
+    REQUIRE(ACGT_TO_IDX('A') != ACGT_TO_IDX('G'));
+    REQUIRE(ACGT_TO_IDX('A') != ACGT_TO_IDX('T'));
+    REQUIRE(ACGT_TO_IDX('C') != ACGT_TO_IDX('G'));
+    REQUIRE(ACGT_TO_IDX('C') != ACGT_TO_IDX('T'));
+    REQUIRE(ACGT_TO_IDX('G') != ACGT_TO_IDX('T'));
+}
+
+
+TEST_CASE("IDX_TO_ACGT", "[fastq::*]")
+{
+    REQUIRE(IDX_TO_ACGT(ACGT_TO_IDX('A')) == 'A');
+    REQUIRE(IDX_TO_ACGT(ACGT_TO_IDX('C')) == 'C');
+    REQUIRE(IDX_TO_ACGT(ACGT_TO_IDX('G')) == 'G');
+    REQUIRE(IDX_TO_ACGT(ACGT_TO_IDX('T')) == 'T');
+}
+
 
 ///////////////////////////////////////////////////////////////////////////////
 // Default constructor
@@ -318,6 +346,27 @@ TEST_CASE("trim_trailing_bases__trim_everything", "[fastq::fastq]")
 }
 
 
+TEST_CASE("trim_trailing_bases__trim_3p__trim_ns", "[fastq::fastq]")
+{
+    fastq record("Rec", "NNATN", "23456");
+    const fastq expected_record("Rec", "NNAT", "2345");
+    const fastq::ntrimmed expected_ntrim(0, 1);
+
+    REQUIRE(record.trim_trailing_bases(true, -1, true) == expected_ntrim);
+    REQUIRE(record == expected_record);
+}
+
+
+TEST_CASE("trim_trailing_bases__trim_3p__trim_trailing_bases", "[fastq::fastq]")
+{
+    const fastq expected_record("Rec", "TN", "!$");
+    const fastq::ntrimmed expected_ntrim(0, 3);
+    fastq record("Rec", "TNANT", "!$#!\"");
+
+    REQUIRE(record.trim_trailing_bases(false, 2, true) == expected_ntrim);
+    REQUIRE(record == expected_record);
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 // trim_windowed_bases
 
@@ -518,6 +567,36 @@ TEST_CASE("trim_windowed_bases__trim_window", "[fastq::fastq]")
 }
 
 
+TEST_CASE("trim_windowed_bases__trim_3p", "[fastq::fastq]")
+{
+    fastq record("Rec", "NNAAAAAAAAATNNNNNNN", "##EEEEEEEEEE#######");
+    const fastq expected_record = fastq("Rec", "NNAAAAAAAAAT", "##EEEEEEEEEE");
+    const fastq::ntrimmed expected_ntrim(0, 7);
+    REQUIRE(record.trim_windowed_bases(false, 10, 5, true) == expected_ntrim);
+    REQUIRE(record == expected_record);
+}
+
+
+TEST_CASE("trim_windowed_bases__trim_3p__trim_ns", "[fastq::fastq]")
+{
+    fastq record("Rec", "NNAAAAAAAAATNNNNNNN", "##EEEEEEEEEE#######");
+    const fastq expected_record = fastq("Rec", "NNAAAAAAAAAT", "##EEEEEEEEEE");
+    const fastq::ntrimmed expected_ntrim(0, 7);
+    REQUIRE(record.trim_windowed_bases(true, 10, 5, true) == expected_ntrim);
+    REQUIRE(record == expected_record);
+}
+
+
+TEST_CASE("trim_windowed_bases__trim_3p__reversed", "[fastq::fastq]")
+{
+    fastq record("Rec", "NNNNNNNTAAAAAAAAANN", "#######EEEEEEEEEE##");
+    const fastq expected_record = fastq("Rec", "NNNNNNNTAAAAAAAAA", "#######EEEEEEEEEE");
+    const fastq::ntrimmed expected_ntrim(0, 2);
+    REQUIRE(record.trim_windowed_bases(true, 10, 5, true) == expected_ntrim);
+    REQUIRE(record == expected_record);
+}
+
+
 ///////////////////////////////////////////////////////////////////////////////
 // Truncate
 
@@ -588,7 +667,7 @@ TEST_CASE("truncate_pos_after_last_base", "[fastq::fastq]")
     // Same behavior as string::substr
     fastq current_record("Rec", "ACTTAG", "12I$12");
     REQUIRE_NOTHROW(current_record.truncate(6));
-    REQUIRE_THROWS_AS(current_record.truncate(7), std::out_of_range);
+    REQUIRE_THROWS_AS(current_record.truncate(7), assert_failed);
 }
 
 
